#!/bin/sh

##################
# read sensor PNcode
##################
. /var/log/fw_sensor_pncode

DEF_GM_LIB_CFG="gmlib.cfg_3m"
DEF_PARAM_CFG="param.cfg_org"


# if the sensor is not support 8M then use default configs
if [ "$FW_SENSOR" == "5O" ]; then
    if [ "`diff /opt/gm/app/gmlib.cfg /opt/gm/app/gmlib.cfg_8m`" != "" ]; then
        cp /opt/gm/app/gmlib.cfg_8m /opt/gm/app/gmlib.cfg
        cp /opt/gm/app/param.cfg_8m /opt/gm/app/param.cfg
    fi
elif [ "`diff /opt/gm/app/gmlib.cfg /opt/gm/app/${DEF_GM_LIB_CFG}`" != "" ]; then
	cp /opt/gm/app/${DEF_GM_LIB_CFG} /opt/gm/app/gmlib.cfg
        cp /opt/gm/app/${DEF_PARAM_CFG} /opt/gm/app/param.cfg
fi

case "$FW_SENSOR" in
	5A)
		# IMX124, 2M 30fps
		video_frontend=imx124_2m30fps
		video_system=NTSC
		;;

	5B)
		# IMX124, 2M 60fps
		video_frontend=imx124_2m60fps
		video_system=NTSC
		;;

	5C)
		# IMX124, 3M 30fps
		video_frontend=imx124_3m30fps
		video_system=NTSC
		;;

	5D)
		# IMX236 2M 30fps
		video_frontend=imx236_2m30fps
		video_system=NTSC
		;;

	5E)
		# IMX236 2M 60fps
		video_frontend=imx236_2m60fps
		video_system=PAL
		;;

	5F)
		# IMX123, 2M 30fps
		video_frontend=imx123_2m30fps
		video_system=NTSC
		;;

	5G)
		# IMX123, 2M 60fps
		video_frontend=imx123_2m60fps
		video_system=NTSC
		;;

	5H)
		# IMX123, 3M 30fps
		video_frontend=imx123_3m30fps
		video_system=NTSC
		;;

	5T)
		# IMX222, 2M 30fps
		video_frontend=imx222
		video_system=NTSC
		;;
	5M)
		# IMX178, 5M 30fps
		video_frontend=imx178
		video_system=NTSC
		;;
	5O)
		# OV10823, 8M 15fps
		video_frontend=ov10823
		video_system=NTSC
		;;

	0[78FfJLs])
		# GM8139 BT1120 page0
		video_frontend=bt1120_2m30fps
		video_system=NTSC
		;;

	1[1358CDFfHhIiJKklmNoOPqrSUuvWxXzZ]|3[Cci])
		# 11:Dongyang 1080p zoom
		# 15:Orion MDIN325A 1080p
		# 18:Semocon Pointing PTZ
        # 1D:E-Ronix 1080p ptz
        # 1F:Ronix 1080p, 1C:Wonwoo 1080p
        # 1f:Ronix no Zoom 1080p, 1G:seyeon seinmedia 1080p
        # 1H:NVP2410 1080p, 13:Dongyang 1080p, 1h:NVP2410 1080p PAL Out
        # 1I:Hankook C-Tech IR PTZ 1080p Wonwoo
        # 1i:Hankook C-Tech PTZ FCB-7500
        # 1J:Hankook C-Tech IR PTZ 1080p LG, 1K:YouGuan IR PTZ 1080p
        # 1k:YouGuan IP PTZ 1080p
        # 1l: KEVIS IR PTZ 1080p
        # 1m: Eyenix EN77x 1080p I2C Control
        # 1N:HD-SDI Video Server 1080p 30fps
        # 1O:Neocom 3xHD to full HD (Pelco-D) NTSC
        # 1o:Neocom 3xHD to full HD (Pelco-D) PAL
        # 1P:HD-SDI Video Server 1080p 25fps
        # 1q:Wonwoo 1080p zoom module Pelco-D (NTSC)
        # 1r:Wonwoo 1080p zoom module Pelco-D (PAL)
        # 1S:Sony zoom FCB-ETH6300 1080p
        # 1U:Ronix 1080p, 1u:Ronix 1080p (support CAM IPS firm-up)
        # 1X:Cynix PTZ 1080p
        # 1x:Cynix Zoom Box 1080p
        # 1z:TBT FCB-EV7500 1080P (IBDR-Color)
        # 1Z:TBT FCB-EV7500 1080P (IBDR-Color)
        # 3C:Eyenix EN77x 1080P I2C Control (Cynix)   
        # 3c:Eyenix EN77x 1080P I2C Control (Paran Tek)
        # 3i:Insight 1080p 60fps

        # GM8139 BT1120 page1
        video_frontend=bt1120_2m30fps
        video_system=NTSC
        ;;

    1[49] | 2[0] | 3[I] )
		# 19:GPI360 Sony FCB7100
        # 20
        # 3I:Insight 1080p 30fps 
        # GM8139 BT1120 page1
        # Wonwoo 2M PTZ 1080p (CbCr Swap)
		video_frontend=bt1120_2m30fps_uv_swap
		video_system=NTSC
		;;
	2[1358CFiJjKNPqrSsUuXx])
		# GM8139 BT1120 2M 60fps
		video_frontend=bt1120_2m60fps
		video_system=NTSC
		;;

	[01]n)
		# GM8139 De-interlaced NTSC BT656
		video_frontend=bt656_d1_p30
		video_system=NTSC
		;;

	[01]p)
		# GM8139 De-interlaced PAL BT656
		video_frontend=bt656_d1_p30
		video_system=PAL
		;;
	0D)
		# FW_SENSOR=0D
		# GM8139 VGA BT656
		video_frontend=bt656_vga_p30
		video_system=NTSC
		;;
	09)
		# FW_SENSOR=09
		# GM8139 384x288
		video_frontend=bt656_384x288_p30
		video_system=NTSC
		;;
	*) # unknown
		echo
		echo "Unknown sensor code($FW_SENSOR)!!!"
		echo
		exit 1

esac

##################
# from vg_boot.sh
##################
/sbin/insmod /opt/gm/modules/frammap.ko
cat /proc/frammap/ddr_info

##################
# from vg_boot.sh
##################

# Support video_front_end
# mt9m034, ar0331, ar0330, ar0835, mt9p031
# ov10823, ov2715, ov9715, ov5653, ov9714
# imx122, imx222, imx138, imx236_2m30fps, imx236_2m60fps, imx238, imx169
# mn34220 bt1120_2m30fps, bt1120_2m30fps_uv_swap, bt1120_2m60fps, bt656_d1_p30, bt656_vga_p30, bt656_384x288_p30

chipver=`head -1 /proc/pmu/chipver`
chipid=`echo $chipver | cut -c 1-4`

if [ "$chipid" != "8139" ] && [ "$chipid" != "8138" ] && [ "$chipid" != "8137" ]; then
	echo "Error! Not support chip version $chipver."
	exit
fi

if [ "$video_system" != "NTSC" ] && [ "$video_system" != "PAL" ]; then
	echo "Invalid argument for NTSC/PAL."
	exit
fi

if [ "$1" != "" ] ; then
	video_frontend=$1
fi

if [ "$video_frontend" != "ov10823" ] && [ "$video_frontend" != "ov2715" ] &&
   [ "$video_frontend" != "ov4689" ] && [ "$video_frontend" != "ov5653" ] &&
   [ "$video_frontend" != "ov9715" ] && [ "$video_frontend" != "ov9714" ] &&
   [ "$video_frontend" != "imx122" ] && [ "$video_frontend" != "imx222" ] &&
   [ "$video_frontend" != "imx136" ] && [ "$video_frontend" != "imx138" ] &&
   [ "$video_frontend" != "imx169" ] && [ "$video_frontend" != "imx238" ] &&
   [ "$video_frontend" != "imx236_2m30fps" ] && [ "$video_frontend" != "imx236_2m60fps" ] &&
   [ "$video_frontend" != "imx123_2m30fps" ] && [ "$video_frontend" != "imx123_2m60fps" ] &&
   [ "$video_frontend" != "imx123_3m30fps" ] &&
   [ "$video_frontend" != "imx124_2m30fps" ] && [ "$video_frontend" != "imx124_2m60fps" ] &&
   [ "$video_frontend" != "imx124_3m30fps" ] &&
   [ "$video_frontend" != "imx178" ] &&
   [ "$video_frontend" != "ar0330" ] && [ "$video_frontend" != "ar0331" ] &&
   [ "$video_frontend" != "ar0835" ] &&
   [ "$video_frontend" != "mt9m034" ] && [ "$video_frontend" != "mt9p031" ] && 
   [ "$video_frontend" != "mn34220" ] && 
   [ "$video_frontend" != "gc1004" ] && 
   [ "$video_frontend" != "bt1120_2m30fps" ] && [ "$video_frontend" != "bt1120_2m30fps_uv_swap" ] &&
   [ "$video_frontend" != "bt1120_2m60fps" ] &&
   [ "$video_frontend" != "bt656_d1_p30" ] && [ "$video_frontend" != "bt656_vga_p30" ] && [ "$video_frontend" != "bt656_384x288_p30" ] 
   then
	echo "Invalid argument for video frontend: $video_frontend"
	exit
fi

/sbin/insmod /opt/gm/modules/log.ko mode=0 log_ksize=256
/sbin/insmod /opt/gm/modules/ms.ko
/sbin/insmod /opt/gm/modules/em.ko

TVOUT_TYPE=`/opt/nss/bin/GetTvOutType`
if [ "$TVOUT_TYPE" == "NTSC" ]  ; then
    echo "Sensor output_type=0, NTSC"
    CVBS_OUT_TYPE="output_type=0"
elif [ "$TVOUT_TYPE" == "PAL" ] ; then
    echo "Sensor output_type=1, PAL"
    CVBS_OUT_TYPE="output_type=1"
else
    echo "[TVOUT_TYPE] ERROR !!!!"
    echo "[TVOUT_TYPE] Use NTSC by default output_type=0"
    CVBS_OUT_TYPE="output_type=0"
    sleep 5
fi
case "$FW_SENSOR" in
    0D)
        /sbin/insmod /opt/gm/modules/flcd200-common_640x480.ko
        /sbin/insmod /opt/gm/modules/flcd200-pip_640x480.ko input_res=2 ${CVBS_OUT_TYPE} fb0_fb1_share=1  
    ;;
    09)
        /sbin/insmod /opt/gm/modules/flcd200-common_360x240.ko
        /sbin/insmod /opt/gm/modules/flcd200-pip_360x240.ko input_res=22 ${CVBS_OUT_TYPE} fb0_fb1_share=1 
    ;;
    *)
        /sbin/insmod /opt/gm/modules/flcd200-common.ko
        /sbin/insmod /opt/gm/modules/flcd200-pip.ko ${CVBS_OUT_TYPE} fb0_fb1_share=1	# CVBS display
    ;;
esac

TVOUT_TYPE=`/opt/nss/bin/GetTvOutType`
if [ "$TVOUT_TYPE" == "NTSC" ]  ; then
    echo 0 > /proc/flcd200/pip/output_type
elif [ "$TVOUT_TYPE" == "PAL" ] ; then
    echo 1 > /proc/flcd200/pip/output_type
else
    echo 0 > /proc/flcd200/pip/output_type
fi
/sbin/insmod /opt/gm/modules/sar_adc.ko run_mode=0x04
/sbin/insmod /opt/gm/modules/think2d.ko
/sbin/insmod /opt/gm/modules/ftpwmtmr010.ko
/sbin/insmod /opt/gm/modules/fe_common.ko
/sbin/insmod /opt/gm/modules/adda302.ko input_mode=0 single_end=1 LIV=-16 LADV=10
/sbin/insmod /opt/gm/modules/ft3dnr.ko src_yc_swap=1 dst_yc_swap=1 ref_yc_swap=1


case "$video_frontend" in
    "imx178") # 3072x2048
        codec_max_width=3072
        codec_max_height=2048
        if [ "$video_system" == "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx178.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx178.ko sensor_w=3072 sensor_h=2048 fps=30 mirror=1
        elif [ "$video_system" == "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx178.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx178.ko sensor_w=3072 sensor_h=2048 fps=30 mirror=1 
        fi
        ;;
    "ov10823")
        codec_max_width=4096
        codec_max_height=2432
        if [ "$video_system" == "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov10823.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ov10823.ko sensor_w=4096 sensor_h=2160 fps=15 mirror=1
        elif [ "$video_system" == "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov10823.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov10823.ko sensor_w=4096 sensor_h=2432 fps=15 mirror=1 
        fi
        ;;
    "ov2715")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov2715.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ov2715.ko sensor_w=1920 sensor_h=1080 interface=0 fps=30 mirror=1 flip=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov2715.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov2715.ko sensor_w=1920 sensor_h=1080 interface=0 fps=25 mirror=1 flip=1
        fi
        ;;

    "ov9715")
        codec_max_width=1280
        codec_max_height=720
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov9715.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ov9715.ko sensor_w=1280 sensor_h=720 fps=30 mirror=1 flip=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov9715.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov9715.ko sensor_w=1280 sensor_h=720 fps=25 mirror=1 flip=1
        fi
        ;;

    "ov5653")
        codec_max_width=2592
        codec_max_height=1944
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov5653.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ov5653.ko sensor_w=2592 sensor_h=1944 interface=1 fps=15 flip=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov5653.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov5653.ko sensor_w=2592 sensor_h=1944 interface=1 fps=15 flip=1
        fi
        ;;

    "ar0331")
        codec_max_width=2048
        codec_max_height=1536
        if [ "$video_system" = "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ar0331.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ar0331.ko sensor_w=2048 sensor_h=1536 interface=2 fps=30 wdr=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ar0331.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ar0331.ko sensor_w=2048 sensor_h=1536 interface=2 fps=25 wdr=1
        fi
        ;;

    "mt9m034")
        codec_max_width=1280
        codec_max_height=960
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_mt9m034.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_mt9m034.ko sensor_w=1280 sensor_h=960 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_mt9m034.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_mt9m034.ko sensor_w=1280 sensor_h=960 fps=25
        fi
        ;;

    "imx122")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx122.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx122.ko sensor_w=1920 sensor_h=1080 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx22.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx122.ko sensor_w=1920 sensor_h=1080 fps=25
        fi
        ;;

    "imx124_2m30fps")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx124.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx124.ko sensor_w=1920 sensor_h=1080 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx124.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx124.ko sensor_w=1920 sensor_h=1080 fps=25
        fi
        ;;

    "imx124_2m60fps")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx124.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx124.ko sensor_w=1920 sensor_h=1080 fps=60
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx124.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx124.ko sensor_w=1920 sensor_h=1080 fps=50
        fi
        ;;

    "imx124_3m30fps")
        codec_max_width=2048
        codec_max_height=1536
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx124.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx124.ko sensor_w=2048 sensor_h=1536 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx124.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx124.ko sensor_w=2048 sensor_h=1536 fps=25
        fi
        ;;

    "imx123_2m30fps")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx123.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx123.ko sensor_w=1920 sensor_h=1080 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx123.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx123.ko sensor_w=1920 sensor_h=1080 fps=25
        fi
        ;;

    "imx123_2m60fps")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx123.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx123.ko sensor_w=1920 sensor_h=1080 fps=60
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx123.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx123.ko sensor_w=1920 sensor_h=1080 fps=50
        fi
        ;;

    "imx123_3m30fps")
        codec_max_width=2048
        codec_max_height=1536
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx123.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx123.ko sensor_w=2048 sensor_h=1536 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx123.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx123.ko sensor_w=2048 sensor_h=1536 fps=25
        fi
        ;;

    "imx136")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx136.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx136.ko sensor_w=1920 sensor_h=1080 fps=60
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx136.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx136.ko sensor_w=1920 sensor_h=1080 fps=50
        fi
        ;;

    "imx138")
        codec_max_width=1280
        codec_max_height=720
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx138.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx138.ko sensor_w=1280 sensor_h=720 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx138.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx138.ko sensor_w=1280 sensor_h=720 fps=25
        fi
        ;;

    "imx169")
        codec_max_width=4096
        codec_max_height=2160
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/app/ko/fisp320.ko cfg_path=/opt/gm/app/isp320_imx169.cfg
            /sbin/insmod /opt/gm/app/ko/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/app/ko/fisp_imx169.ko sensor_w=4096 sensor_h=2160 fps=15
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/app/ko/fisp320.ko cfg_path=/opt/gm/app/isp320_imx169.cfg
            /sbin/insmod /opt/gm/app/ko/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/app/ko/fisp_imx169.ko sensor_w=4096 sensor_h=2160 fps=15
        fi
        ;;

    "imx238")
        codec_max_width=1280
        codec_max_height=720
        if [ "$video_system" = "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx238.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx138.ko sensor_w=1280 sensor_h=720 fps=30 sensor_spi=1
        elif [ "$video_system" = "PAL" ] ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx238.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx138.ko sensor_w=1280 sensor_h=720 fps=25 sensor_spi=1
        fi
        ;;

    "imx222")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx222.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx122.ko sensor_w=1920 sensor_h=1080 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx222.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx122.ko sensor_w=1920 sensor_h=1080 fps=25
        fi
        ;;

    "imx236_2m30fps")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx236.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx136.ko sensor_w=1920 sensor_h=1080 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx236.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx136.ko sensor_w=1920 sensor_h=1080 fps=25
        fi
        ;;

    "imx236_2m60fps")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx236.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_imx136.ko sensor_w=1920 sensor_h=1080 fps=60
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_imx236.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_imx136.ko sensor_w=1920 sensor_h=1080 fps=50
        fi
        ;;

    "ar0835")
        codec_max_width=3264
        codec_max_height=2448
        if [ "$video_system" = "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ar0835.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ar0835.ko sensor_w=3264 sensor_h=2448 fps=15 flip=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ar0835.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ar0835.ko sensor_w=3264 sensor_h=2448 fps=15 flip=1
        fi
        ;;

    "ar0330")
        codec_max_width=2048;codec_max_height=1536
        #codec_max_width=2304;codec_max_height=1296
        if [ "$video_system" = "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ar0330.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ar0330.ko sensor_w=2048 sensor_h=1536 fps=30 flip=1 mirror=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ar0330.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ar0330.ko sensor_w=2048 sensor_h=1536 fps=25 flip=1 mirror=1
        fi
        ;;

    "ov4689")
        codec_max_width=1920
        codec_max_height=1080
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov4689.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov4689.ko sensor_w=1920 sensor_h=1080 fps=30 flip=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov4689.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov4689.ko sensor_w=1920 sensor_h=1080 fps=25 flip=1
        fi
        ;;

    "gc1004")
        codec_max_width=2048;codec_max_height=1280
        #codec_max_width=2304;codec_max_height=720
        if [ "$video_system" = "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_gc1004.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_gc1004.ko sensor_w=2048 sensor_h=1024 fps=30 flip=1 mirror=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_gc1004.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_gc1004.ko sensor_w=2048 sensor_h=1024 fps=25 flip=1 mirror=1
        fi
        ;;

    "mt9p031")
        codec_max_width=2592;codec_max_height=1944
        #codec_max_width=2304;codec_max_height=720
        if [ "$video_system" = "NTSC" ]  ; then
        	/sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_mt9p031.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_mt9p031.ko sensor_w=2592 sensor_h=1944 fps=10 flip=0 mirror=0
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_mt9p031.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_mt9p031.ko sensor_w=2592 sensor_h=1944 fps=10 flip=0 mirror=0
        fi
        ;;

    "ov9714")
        codec_max_width=1280
        codec_max_height=720
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov9714.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_ov9714.ko sensor_w=1280 sensor_h=720 fps=30 flip=1 mirror=1
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_ov9714.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_ov9714.ko sensor_w=1280 sensor_h=720 fps=25 flip=1 mirror=1  
        fi
        ;;
    "mn34220")
        codec_max_width=1920;codec_max_height=1080  
        if [ "$video_system" = "NTSC" ]  ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_mn34220.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=0
            /sbin/insmod /opt/gm/modules/fisp_mn34220.ko sensor_w=1920 sensor_h=1080 fps=30
        elif [ "$video_system" = "PAL" ] ; then
            /sbin/insmod /opt/gm/modules/fisp320.ko cfg_path=/opt/gm/app/isp320_mn34220.cfg
            /sbin/insmod /opt/gm/modules/fisp_algorithm.ko pwr_freq=1
            /sbin/insmod /opt/gm/modules/fisp_mn34220.ko sensor_w=1920 sensor_h=1080 fps=25
        fi
        ;;

    "bt1120_2m30fps")
        codec_max_width=1920;codec_max_height=1080
        ;;

    "bt1120_2m30fps_uv_swap")
        codec_max_width=1920;codec_max_height=1080
        ;;

    "bt1120_2m60fps")
        codec_max_width=1920;codec_max_height=1080
        ;;

    "bt656_d1_p30")
        if [ "$video_system" = "NTSC" ]  ; then
        	codec_max_width=720;codec_max_height=480
        elif [ "$video_system" = "PAL" ] ; then
        	codec_max_width=720;codec_max_height=576
		fi
        ;;
    "bt656_vga_p30") # FW_SENSOR=0D
		if [ "$video_system" = "NTSC" ]  ; then
        	codec_max_width=640;codec_max_height=480
        # elif [ "$video_system" = "PAL" ] ; then
        # 	codec_max_width=720;codec_max_height=576
		fi
        ;;
    "bt656_384x288_p30") # FW_SENSOR=09
		if [ "$video_system" = "NTSC" ]  ; then
        	codec_max_width=384;codec_max_height=288
        # elif [ "$video_system" = "PAL" ] ; then
        # 	codec_max_width=720;codec_max_height=576
		fi
        ;;
    *)
        echo "Invalid argument for video frontend: $video_frontend"
        exit
        ;;
esac

/sbin/insmod /opt/gm/modules/vcap300_common.ko
if [ "$video_frontend" = "bt1120_2m30fps" ]  ; then
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=1 ext_irq_src=1
	if [ "$video_system" = "NTSC" ]  ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=1920 height=1080 interface=3 mode=0 order=0 yc_swap=0 inv_clk=0 frame_rate=30 speed=1 timeout=66 ch_id=0
		echo "vcap300_generic_m0 : bt1120_2m30fps/NTSC"
	elif [ "$video_system" = "PAL" ] ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=1920 height=1080 interface=3 mode=0 order=0 yc_swap=0 inv_clk=0 frame_rate=25 speed=1 timeout=80 ch_id=0
		echo "vcap300_generic_m0 : bt1120_2m25fps/PAL"
	fi
elif [ "$video_frontend" = "bt1120_2m30fps_uv_swap" ]  ; then
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=1 ext_irq_src=1
	if [ "$video_system" = "NTSC" ]  ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=1920 height=1080 interface=3 mode=0 order=0 yc_swap=2 inv_clk=0 frame_rate=30 speed=1 timeout=66 ch_id=0
		echo "vcap300_generic_m0 : bt1120_2m30fps_uv_swap/NTSC"
	elif [ "$video_system" = "PAL" ] ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=1920 height=1080 interface=3 mode=0 order=0 yc_swap=2 inv_clk=0 frame_rate=25 speed=1 timeout=80 ch_id=0
		echo "vcap300_generic_m0 : bt1120_2m25fps/PAL"
	fi
elif [ "$video_frontend" = "bt1120_2m60fps" ]  ; then
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=1 ext_irq_src=1
	if [ "$video_system" = "NTSC" ]  ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=1920 height=1080 interface=3 mode=0 order=0 yc_swap=0 inv_clk=0 frame_rate=60 speed=1 timeout=66 ch_id=0
		echo "vcap300_generic_m0 : bt1120_2m60fps/NTSC"
	elif [ "$video_system" = "PAL" ] ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=1920 height=1080 interface=3 mode=0 order=0 yc_swap=0 inv_clk=0 frame_rate=50 speed=1 timeout=80 ch_id=0
		echo "vcap300_generic_m0 : bt1120_2m50fps/PAL"
	fi
elif [ "$video_frontend" = "bt656_d1_p30" ]  ; then
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=1 ext_irq_src=1
	if [ "$video_system" = "NTSC" ]  ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=384 height=288 interface=1 mode=0 order=0 yc_swap=0 inv_clk=1 frame_rate=30 speed=1 timeout=66 ch_id=0
		echo "vcap300_generic_m0 : bt656_d1_p30/NTSC"
	elif [ "$video_system" = "PAL" ] ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=720 height=576 interface=1 mode=0 order=0 yc_swap=0 inv_clk=1 frame_rate=25 speed=1 timeout=80 ch_id=0
		echo "vcap300_generic_m0 : bt656_d1_p30/PAL"
	fi
elif [ "$video_frontend" = "bt656_vga_p30" ]  ; then 
# FW_SENSOR=0D
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=1 ext_irq_src=1
	if [ "$video_system" = "NTSC" ]  ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=640 height=480 interface=1 mode=0 order=0 yc_swap=0 inv_clk=1 frame_rate=30 speed=1 timeout=66 ch_id=0
		echo "vcap300_generic_m0 : bt656_vga_p30/NTSC"
	# elif [ "$video_system" = "PAL" ] ; then
	# 	/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=720 height=576 interface=1 mode=0 order=0 yc_swap=0 inv_clk=1 frame_rate=25 speed=1 timeout=80 ch_id=0
	# 	echo "vcap300_generic_m0 : bt656_vga_p30/PAL"
	fi
elif [ "$video_frontend" = "bt656_384x288_p30" ]  ; then 
# FW_SENSOR=09
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=1 ext_irq_src=1
	if [ "$video_system" = "NTSC" ]  ; then
		/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=384 height=288 interface=1 mode=0 order=0 yc_swap=0 inv_clk=1 frame_rate=30 speed=1 timeout=66 ch_id=0
		echo "vcap300_generic_m0 : bt656_384x288_p30/NTSC"
	# elif [ "$video_system" = "PAL" ] ; then
	# 	/sbin/insmod /opt/gm/modules/vcap300_generic_m0.ko width=384 height=384 interface=1 mode=0 order=0 yc_swap=0 inv_clk=1 frame_rate=25 speed=1 timeout=80 ch_id=0
	# 	echo "vcap300_generic_m0 : bt656_384x288_p30/PAL"
	fi
else
	/sbin/insmod /opt/gm/modules/fisp_iris_pwm.ko
	/sbin/insmod /opt/gm/modules/vcap0.ko vi_mode=0,1 ext_irq_src=1
	/sbin/insmod /opt/gm/modules/vcap300_isp.ko ch_id=0 range=1
fi

/sbin/insmod /opt/gm/modules/fmcp_drv.ko mp4_tight_buf=1
/sbin/insmod /opt/gm/modules/favc_enc.ko h264e_max_b_frame=0 h264e_one_ref_buf=1 h264e_tight_buf=1 h264e_max_width=$codec_max_width h264e_max_height=$codec_max_height h264e_slice_offset=1
/sbin/insmod /opt/gm/modules/favc_rc.ko
/sbin/insmod /opt/gm/modules/decoder.ko
/sbin/insmod /opt/gm/modules/fmjpeg_drv.ko
#/sbin/insmod /opt/gm/modules/fmpeg4_drv.ko mp4_max_width=$codec_max_width mp4_max_height=$codec_max_height
#/sbin/insmod /opt/gm/modules/mp4e_rc.ko

# Encode 4CH + Cascade YUV 1CH
/sbin/insmod /opt/gm/modules/sw_osg.ko
/sbin/insmod /opt/gm/modules/fscaler300.ko max_vch_num=5 max_minors=5 temp_width=0 temp_height=0
#/sbin/insmod /opt/gm/modules/ftdi220.ko
/sbin/insmod /opt/gm/modules/osd_dispatch.ko
/sbin/insmod /opt/gm/modules/codec.ko
/sbin/insmod /opt/gm/modules/audio_drv.ko audio_ssp_num=0,1 audio_ssp_chan=1,1 bit_clock=400000,400000 sample_rate=8000,8000 audio_out_enable=1,1
/sbin/insmod /opt/gm/modules/gs.ko reserved_ch_cnt=2 alloc_unit_size=262144 flow_mode=1
/sbin/insmod /opt/gm/modules/loop_comm.ko
/sbin/insmod /opt/gm/modules/vpd_slave.ko vpslv_dbglevel=0 ddr0_sz=0 ddr1_sz=0 config_path="/opt/gm/app" usr_func=0 usr_param=0 datain_minors=4 dataout_minors=8
/sbin/insmod /opt/gm/modules/vpd_master.ko vpd_dbglevel=0 gmlib_dbglevel=0

echo /mnt/nfs > /proc/videograph/dumplog   #configure log path
mdev -s
cat /proc/modules
#echo 0 > /proc/frammap/free_pages   #should not free DDR1 for performance issue
echo 1 > /proc/vcap300/vcap0/dbg_mode  #need debug mode to detect capture overflow
echo 0 > /proc/videograph/em/dbglevel
echo 0 > /proc/videograph/gs/dbglevel
echo 0 > /proc/videograph/ms/dbglevel
echo 0 > /proc/videograph/datain/dbglevel
echo 0 > /proc/videograph/dataout/dbglevel
echo 0 > /proc/videograph/vpd/dbglevel
echo 0 > /proc/videograph/gmlib/dbglevel


echo =========================================================================
if [ -e /opt/gm/app/gmlib.cfg ] ; then
	grep ";" /opt/gm/app/gmlib.cfg |sed -n '1,1p'
else
	grep ";" /opt/gm/app/spec.cfg |sed -n '2,6p'
fi

##rootfs_date=`ls /|grep 00_2`
##mtd_date=`ls /mnt/mtd|grep 00_2`
echo =========================================================================
echo "  Video Front End: $video_frontend"
echo "  Chip Version: $chipver"
##echo "  RootFS Version: $rootfs_date"
##echo "  MTD Version: $mtd_date"
echo =========================================================================

devmem 0x9a100030 32 0x9f000009    # channel arbitration, increase vcap & CPU
devmem 0x9a100034 32 0x08080606    # channel arbitration grant count for vcap & codec
devmem 0x9a1000a0 32 0x87878587    # AHB read prefetch length for USB dongle


#devmem 0x96105440 32 0x01500000
#devmem 0x96105438 32 0x01500000
#echo 1 0x50 > /proc/3dnr/dma/param
#echo w ae_en 0 > /proc/isp320/command
#echo w sen_exp 133 > /proc/isp320/command
#echo w fps 15 > /proc/isp320/command

# force max CPU performance
#echo performance > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor

